pub enum Config {
Ticket(String),
TicketSender(Sender<String>),
}Expand description
Config used to create a new NetcodeInterface.
The user was either given a ticket, or is generating a new ticket.
Variants
Ticket(String)
A ticket string obtained from the other player.
TicketSender(Sender<String>)
Holds a oneshot sender that will send a newly generated ticket.